@@ -94,20 +94,24 @@ G_DEFINE_TYPE (GSJob, gs_job, G_TYPE_OBJECT);
 static xmlXPathObjectPtr
 getnodeset (xmlDocPtr      doc,
             const xmlChar *xpath)
-{	
-	xmlXPathContextPtr context;
-	xmlXPathObjectPtr  result;
+{
+        xmlXPathContextPtr context;
+        xmlXPathObjectPtr  result;
 
-	context = xmlXPathNewContext (doc);
-	result = xmlXPathEvalExpression (xpath, context);
-	if (xmlXPathNodeSetIsEmpty (result->nodesetval)) {
+        context = xmlXPathNewContext (doc);
+        result = xmlXPathEvalExpression (xpath, context);
+        xmlXPathFreeContext (context);
+
+        if (! result)
+                return NULL;
+
+        if (xmlXPathNodeSetIsEmpty (result->nodesetval)) {
                 g_warning ("Node set is empty for %s", xpath);
-		return NULL;
+                xmlXPathFreeObject (result);
+                return NULL;
         }
 
-	xmlXPathFreeContext (context);
-
-	return result;
+        return result;
 }
 
 static xmlChar *
@@ -116,15 +120,17 @@ get_first_xpath_prop (xmlDocPtr      doc,
                       const xmlChar *prop)
 { 
         xmlChar          *keyword = NULL;
-	xmlXPathObjectPtr result;
-	xmlNodeSetPtr     nodeset;
+        xmlXPathObjectPtr result;
+        xmlNodeSetPtr     nodeset;
 
-	result = getnodeset (doc, xpath);
+        result = getnodeset (doc, xpath);
         if (result) {
-		nodeset = result->nodesetval;
+                nodeset = result->nodesetval;
                 if (nodeset->nodeNr > 0)
                         keyword = xmlGetProp (nodeset->nodeTab [0], prop);
-	}
+
+                xmlXPathFreeObject (result);
+        }
 
         return keyword;
 }
@@ -185,6 +191,21 @@ gs_job_get_theme_path (GSJob *job,
         }
 }
 
+void
+gs_job_append_theme_path (GSJob      *job,
+                          const char *path)
+{
+        g_return_if_fail (GS_IS_JOB (job));
+        g_return_if_fail (path != NULL);
+
+        job->priv->search_path_len++;
+        job->priv->search_path = g_renew (char *, job->priv->search_path, job->priv->search_path_len);
+
+        job->priv->search_path [job->priv->search_path_len - 1] = g_strdup (path);
+
+        /*do_theme_change (job);*/
+}
+
 void
 gs_job_prepend_theme_path (GSJob      *job,
                            const char *path)
@@ -354,6 +375,9 @@ load_themes (GSJob *job)
                         char           *title;
                         char          **argv;
 
+                        if (! g_str_has_suffix (file, ".xml"))
+                                continue;
+
                         path = g_build_filename (dir, file, NULL);
 
                         if (! parse_theme (path,
@@ -541,6 +565,11 @@ gs_job_init (GSJob *job)
         i = 0;
         job->priv->search_path [i++] = g_strdup (THEMESDIR);
 
+#ifdef XSCREENSAVER_CONFIG_DIR
+        if (g_file_test (XSCREENSAVER_CONFIG_DIR, G_FILE_TEST_IS_DIR))
+                gs_job_append_theme_path (job, XSCREENSAVER_CONFIG_DIR);
+#endif
+
         job->priv->themes_valid = FALSE;
 }
 
@@ -610,6 +639,9 @@ gs_job_finalize (GObject *object)
         g_free (job->priv->search_path);
         job->priv->search_path = NULL;
 
+        if (job->priv->all_themes)
+                g_hash_table_destroy (job->priv->all_themes);
+
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -625,6 +657,9 @@ gs_job_set_widget  (GSJob     *job,
 
 static const char *known_locations [] = {
         SAVERDIR,
+#ifdef XSCREENSAVER_HACK_DIR
+        XSCREENSAVER_HACK_DIR,
+#endif
         LIBEXECDIR "/xscreensaver",
         "/usr/X11R6/lib/xscreensaver",
         "/usr/libexec/xscreensaver",
@@ -679,26 +714,31 @@ gs_job_set_theme  (GSJob      *job,
         GSJobThemeInfo *info;
 
         g_return_val_if_fail (GS_IS_JOB (job), FALSE);
-        g_return_val_if_fail (theme != NULL, FALSE);
 
-        info = gs_job_lookup_theme_info (job, theme);
+        /* NULL theme is interpreted as a no-op job */
 
-        if (! info) {
-                /* FIXME: set error */
-                g_warning ("Could not lookup info for theme: %s", theme);
-                return FALSE;
+        if (theme) {
+                info = gs_job_lookup_theme_info (job, theme);
+
+                if (! info) {
+                        /* FIXME: set error */
+                        g_warning ("Could not lookup info for theme: %s", theme);
+                        return FALSE;
+                }
+
+                if (! check_command (info->argv)) {
+                        /* FIXME: set error */
+                        g_warning ("Could not verify safety of command for theme: %s", theme);
+                        return FALSE;
+                }
+
+                gs_job_theme_info_free (info);
         }
 
-        if (! check_command (info->argv)) {
-                /* FIXME: set error */
-                g_warning ("Could not verify safety of command for theme: %s", theme);
-                return FALSE;
-        }
-
-        gs_job_theme_info_free (info);
-
         g_free (job->priv->current_theme);
-        job->priv->current_theme = g_strdup (theme);
+
+        if (theme)
+                job->priv->current_theme = g_strdup (theme);
 
         return TRUE;
 }
@@ -752,18 +792,17 @@ spawn_on_widget (GtkWidget  *widget,
                  gpointer    user_data,
                  guint      *watch_id)
 {
-        char       *envp [5];
         char       *path;
         char      **new_argv;
-        int         nenv = 0;
-        int         i;
-        char       *window_id;
+        GPtrArray  *env;
+        char       *str;
         gboolean    result;
         GIOChannel *channel;
         GError     *error = NULL;
         int         standard_error;
         int         child_pid;
         int         id;
+        int         i;
 
         if (! argv)
                 return FALSE;
@@ -776,19 +815,26 @@ spawn_on_widget (GtkWidget  *widget,
                 new_argv [0] = path;
         }
 
-        window_id = widget_get_id_string (widget);
-        envp [nenv++] = g_strdup_printf ("XSCREENSAVER_WINDOW=%s", window_id);
-        envp [nenv++] = g_strdup_printf ("DISPLAY=%s",
-                                         gdk_display_get_name (gdk_display_get_default ()));
-        envp [nenv++] = g_strdup_printf ("HOME=%s",
-                                         g_get_home_dir ());
-        envp [nenv++] = g_strdup_printf ("PATH=%s", g_getenv ("PATH"));
-        envp [nenv++] = NULL;
+        env = g_ptr_array_new ();
+
+        str = widget_get_id_string (widget);
+        g_ptr_array_add (env, g_strdup_printf ("XSCREENSAVER_WINDOW=%s", str));
+        g_free (str);
+
+        str = gdk_screen_make_display_name (gtk_widget_get_screen (widget));
+        g_ptr_array_add (env, g_strdup_printf ("DISPLAY=%s", str));
+        g_free (str);
+
+        g_ptr_array_add (env, g_strdup_printf ("HOME=%s",
+                                               g_get_home_dir ()));
+        g_ptr_array_add (env, g_strdup_printf ("PATH=%s", g_getenv ("PATH")));
+        g_ptr_array_add (env, g_strdup_printf ("XAUTHORITY=%s", g_getenv ("XAUTHORITY")));
+        g_ptr_array_add (env, NULL);
 
         result = gdk_spawn_on_screen_with_pipes (gtk_widget_get_screen (widget),
                                                  g_get_home_dir (),
                                                  new_argv,
-                                                 envp,
+                                                 (char **)env->pdata,
                                                  G_SPAWN_DO_NOT_REAP_CHILD,
                                                  NULL,
                                                  NULL,
@@ -797,13 +843,19 @@ spawn_on_widget (GtkWidget  *widget,
                                                  NULL,
                                                  &standard_error,
                                                  &error);
+        for (i = 0; i < env->len; i++)
+                g_free (g_ptr_array_index (env, i));
+        g_ptr_array_free (env, TRUE);
 
         if (! result) {
                 g_warning ("Could not start command '%s': %s", new_argv [0], error->message);
                 g_error_free (error);
+                g_strfreev (new_argv);
                 return FALSE;
         }
 
+        g_strfreev (new_argv);
+
         nice_process (child_pid, 10);
 
         if (pid)
@@ -825,9 +877,6 @@ spawn_on_widget (GtkWidget  *widget,
 
         g_io_channel_unref (channel);
 
-        for (i = 0; i < nenv; i++)
-                g_free (envp [i]);
-
         return result;
 }
 
@@ -880,13 +929,14 @@ gs_job_start (GSJob *job)
                 return FALSE;
         }
 
-        if (! job->priv->current_theme) {
-                g_warning ("Could not start job: screensaver theme is not set.");
+        if (! job->priv->widget) {
+                g_warning ("Could not start job: screensaver window is not set.");
                 return FALSE;
         }
 
-        if (! job->priv->widget) {
-                g_warning ("Could not start job: screensaver window is not set.");
+        if (! job->priv->current_theme) {
+                /* no warning here because a NULL theme is interpreted
+                   as a no-op job */
                 return FALSE;
         }
 
